#!/usr/local/bin/perl5.00502
#               -------------
#                   Links
#               -------------
#               Links Manager
#
#         File: build.pl
#  Description: This routine validates the links. It checks to see that a return code of 
#               200 is returned for each link and flags any errors. Return codes of 404 (File
#               Not Found) should be updated. Return codes of 500, should be checked but might still 
#               be valid. It will store the results in a log file specified in the config file.
#
#               NOTE: This routine requires the LWP module and NET::FTP available from 
#					   CPAN: http://perl.com/CPAN/modules/by-module/LWP/
#
#       Author: Alex Krohn
#        Email: alex@gossamer-threads.com
#          Web: http://www.gossamer-threads.com/
#      Version: 2.0b2
#
# (c) 1998 Gossamer Threads Inc. 
#
# This script is not freeware! Please read the README for full details
# on registration and terms of use. 
# =====================================================================

package Validator;
# ===============================================================
# We build our own user agent off of Parallel User Agent.
#
	BEGIN {
		($0 =~ m,(.*)/[^/]+,)   && unshift (@INC, "$1");	# Get the script location: UNIX /
		($0 =~ m,(.*)\\[^\\]+,) && unshift (@INC, "$1");	# Get the script location: Windows \
	}	
	eval {
		require 5.004;
		use Exporter();
		use LWP::Parallel::UserAgent qw(:CALLBACK);
		use vars qw(@ISA @EXPORT $VERSION);
		$VERSION = '1.0';

		@ISA = qw(LWP::Parallel::UserAgent Exporter);
		@EXPORT = @LWP::Parallel::UserAgent::EXPORT_OK;
	};
	$@ and &nph_err($@);

sub on_failure {
# ---------------------------------------------------------------
# Get's called whenever we fail right away.
#
	my ($self, $request, $response, $entry) = @_;
	if ($response) {
		my $url  = $request->url;
		my $id   = $::urls{$url};
		my $code = $response->code;
		my $msg  = $response->message;
		chomp ($msg); chomp ($url);
		
		$::code{$url} = $code;
		$::msg {$url} = $msg;
		
		$ENV{'REQUEST_METHOD'} ?
			print qq|Checked <a href="$url" target="_blank">$id</a> - Request Failed ($code). Message: $msg\n| :
			print qq|Checked $id - Request Failed ($code). Message: $msg\n|;			
	}
}

sub on_return {
# ---------------------------------------------------------------
# Get's called for every terminated HTTP connection.
#
    my ($self, $request, $response, $entry) = @_;
	if ($response) {
		my $url  = $request->url;
		my $id   = $::urls{$url};
		my $code = $response->code;
		my $msg  = $response->message;
		chomp ($msg); chomp ($url);
		
		if ($response->is_success) {
			$ENV{'REQUEST_METHOD'} ?
				print qq|Checked <a href="$url" target="_blank">$id</a> - Success ($code). Message: $msg. URL: $url\n| :
				print qq|Checked $id - Success ($code). Message: $msg\n|;
		}
		else {
			$::code{$url} = $code;
			$::msg {$url} = $msg;
			$ENV{'REQUEST_METHOD'} ?
				print qq|Checked <a href="$url" target="_blank">$id</a> - Request Failed ($code). Message: $msg. URL: $url\n| :
				print qq|Checked $id - Request Failed ($code). Message: $msg\n|;			
		}		
	}
}

package main;
# ===============================================================

# Required Librariers
# --------------------------------------------------------
	eval {
		require 5.004;
		require "links.cfg";				# Change this to full path to links.cfg if you have problems.		
		require "$db_lib_path/db_utils.pl";
		require "$db_lib_path/links.def";		
		use HTTP::Request;
#		use LWP::Debug qw(+);				# Turn on for copious LWP debugging info.
	};
	$@ and &nph_err($@);

# ========================================================
eval    { &verify_links; };					# Trap any fatal errors so the program hopefully 
$@  and &cgierr("Fatal error: $@");         # never produces that nasty 500 server error page.
exit;   
# ========================================================

sub verify_links {
# -----------------------------------------------------
    $|++;
			
	my $use_html = $ENV{'REQUEST_METHOD'};;
	my $start    = time();
	local %urls  = ();
	local %code  = ();
	local %msg   = ();

# Print headers.	
	$nph++;
	$use_html and &html_print_headers(1);
	$use_html ?
    	print qq|
			<html><head><tittle>Links Manager: Verifying Links</title></head>
			<BODY BGCOLOR=#FFFFFF><H2><TT>Verifying Links</TT></H2>
			<p>Link checking started at: |, scalar localtime($start), qq|<pre>
--------------------------------------------------------
Registering URL's ... \n| :
		print "Link Checking stated: ", scalar localtime($start), "\nRegistering URL's ... \n";

# Set up the link validator.
	my $checker = new Validator;

# Using a proxy? You should have the environment variable HTTP_proxy set to your
# proxy server, or try manually by setting: 
#	$ENV{'HTTP_PROXY'} = "http://host";
	$checker->env_proxy();				# Set up any proxy settings from environment.	

# Please do not change the user agent.
	$checker->agent('Links - http://gossamer-threads.com/scripts/links/'); 
	
# Set some other options.	
	$checker->remember_failures(1);     # If it's bad once, it's bad.
	$checker->max_req(5);               # Tinker for performance. Number of requests to check in parallel.
	$checker->redirect(0);              # Should we follow redirects? Default is no.
	$checker->in_order(1);				# Keep the order.
	$checker->timeout(5);				# Number of seconds to wait.

# Use HEAD for quick checks, GET for full checks.
	(($ENV{'QUERY_STRING'} eq 'detailed') or (@ARGV > -1 and $ARGV[0] eq 'detailed')) ?
		($method = 'GET') :
		($method = 'HEAD');	

# Go through and register all the URL's.	
	my (@data, $total, $request, %seen);
	
    open (DB, "<$db_file_name") or &cgierr("error in verify_links. unable to open db file: $db_file_name. Reason: $!");
	while (<DB>) {
		/^#/      and next;		# Skip comment Lines.
		/^\s*$/   and next;		# Skip blank lines.
		chomp;
		@data = &split_decode($_);
		if (($data[$db_url] =~ /^http/) or ($data[$db_url] =~ /^ftp/)) {
			$seen{$data[$db_url]}++ and next;
			$request             = HTTP::Request->new('GET', $data[$db_url]);
			$urls{$request->url} = $data[$db_key_pos];
			$checker->register ($request);
			$total++;
			!($total % 100)  and print "$total ";
			!($total % 1000) and print "$total\n";			
		}
		last if $total > 100;
	}
    close DB;

# All done registering. Now let's look them up!
	print "\nDone.\n\nLinks to check '$total' using $method method. Please be patient... \n\n";
	my $entries = $checker->wait(); # responses will be caught by on_return, etc

# All done. Print out finish time and summary stats.
	my $finish  = time();
	my $elapsed = $finish - $start;
	print "\nTook: $elapsed seconds to check $total links.\n";

	my ($url, $code, $msg, $id, $badcount);
	print "\nBad Link Summary\n-----------------------------------------------\n";
	foreach $url (sort { $code{$b} <=> $code{$a} } keys %code) {
		$code  = $code{$url};
		$msg   = $msg{$url};
		$id    = $urls{$url};		
		$use_html ?
			print qq~$id - <a href="$url" target="_blank">$url</a> <font size=-1>[<a href="$db_script_url?db=links&modify_form=1&$db_key=$id&ww=1" target="_blank">modify</a>|<a href="$db_script_url?db=links&delete_form=1&$db_key=$id&ww=1" target="_blank">delete</a>]</font> : $code - $msg\n~ :
			print qq~$id - $url : $code - $msg\n~;
		$badcount++;
	}
	print "\n\nGood Links: ", $total - $badcount;
	print "\nBad Links: $badcount\n";
}

sub nph_err {
# -----------------------------------------------------
# Special err for nph script.
#
	my $err = shift;
	
	print "HTTP 1.0/200 OK\n";
    print "Content-type: text/plain\n\n";
	print "Error including libraries: $err\n";
    print "Make sure they exist, permissions are set properly, and paths are set correctly.";
	exit;
}
